<?php

namespace SM\Service;

use SM\Factory\CdaFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use SM\Service\FindService;
use SM\Service\ProcedureService;

class CdaService
{
    protected $CdaFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;
    protected $ProcedureService;

    public function __construct(EntityManager $entityManager, CdaFactory $CdaFactory, Environment $Environment, NewCode $newCode, ProcedureService $ProcedureService)
    {
        $this->CdaFactory                   = $CdaFactory;
        $this->connection                   = $entityManager->getConnection();
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->ProcedureService             = $ProcedureService;
    }

    public function getOperadorLogado()
    {
        return array(
            "CDOPERADOR" => $this->Environment->getCdOperador(),
        );
    }

    public function getCda($params)
    {
        $data = $this->CdaFactory->getCda(array(
            "CDFUNC"            => $params["CDFUNC"],
            "CDFILIAL"          => $params["CDFILIAL"],
            "CDEMPRESA"         => $params["CDEMPRESA"],
            "NRORG"             => $params["NRORG"],
            "CDOPERADOR"        => $params["CDOPERADOR"],
            "DTINICIALPERIODO"  => $params["DTINICIALPERIODO"],
            "DTFINALPERIODO"    => $params["DTFINALPERIODO"]
        ));

        foreach ($data['data'] as &$cda) {
            if (($cda['IDSERVPREST'] == 'ATIVIDADE_CLIENTE' || $cda['IDSERVPREST'] == 'RELACIONAMENTO_CLIENTE') && empty($cda['CDINTECLIE']) && !empty($cda['NMINTCLIENTE'])) {
                $cda['CDINTECLIE'] = $this->CdaFactory->getCodInterlocutorCliente(array(
                    "CDCLIENTE"    => $cda["CDCLIENTE"],
                    "NMINTECLIE"   => $cda["NMINTCLIENTE"]
                ));
            } elseif ($cda['IDSERVPREST'] == 'VISITA_PROSPECT' && empty($cda['CDINTEPROS']) && !empty($cda['NMINTCLIENTE'])) {
                $cda['CDINTEPROS'] = $this->CdaFactory->getCodInterlocutorProspect(array(
                    "CDPROSPECT"    => $cda["CDPROSPECT"],
                    "NMINTEPROS"   => $cda["NMINTCLIENTE"]
                ));
            }
        }

        return $data;
    }

    public function getFuncionarios()
    {
        return $this->CdaFactory->getFuncionarios();
    }

    public function getPermissaoOperadorCda()
    {
        return $this->CdaFactory->getPermissaoOperadorCda(array(
            "CDOPERADOR"    => $this->Environment->getCdOperador(),
            "NRORG"         => $this->Environment->getNrorg(),
        ));
    }

    public function getPeriodoApuraPonto()
    {
        return $this->CdaFactory->getPeriodoApuraPonto();
    }

    public function getClienteCda($parametros)
    {
        return $this->CdaFactory->getClienteCda(array(
            "DTSERVPREST" => $parametros["DTSERVPREST"],
            "CDFUNC"      => $parametros["CDFUNC"],
        ));
    }

    public function getClienteRelacionamento()
    {
        $clientes = $this->CdaFactory->getClienteRelacionamento();
        foreach ($clientes['data'] as $key => $cliente) {
            if ($cliente['IDCLIEATIV'] === 'S') {
                $cliente['IDCLIEATIV'] = 0;
            } else {
                $cliente['IDCLIEATIV'] = 1;
            }
            $clientes['data'][$key] = $cliente;
        }

        return $clientes;
    }

    public function getPropostaCda($params)
    {
        $nrorg = $this->Environment->getNrorg();
        $dataParams = array(
            "CDCLIENTE" => isset($params["CDCLIENTE"]) ? $params["CDCLIENTE"] : null,
            "CDFUNC"    => isset($params["CDFUNC"]) ? $params["CDFUNC"] : null,
            "NRORG"     => $nrorg
        );
        return $this->CdaFactory->getPropostaCda($dataParams);
    }

    public function getProjetoOperadorCliente($params)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $nrorg = $this->Environment->getNrorg();

        $dataParams = array(
            "CDCLIENTE"     => isset($params["CDCLIENTE"]) ? $params["CDCLIENTE"] : null,
            "CDOPERADOR"    => $cdoperador,
            "NRORG"         => $nrorg,
        );
        return $this->CdaFactory->getProjetoOperadorCliente($dataParams);
    }

    public function getTipoAtividadeCda($params)
    {
        $dataParams = array(
            "CDCLIENTE"     => isset($params["CDCLIENTE"]) ? $params["CDCLIENTE"] : null,
            "NRORG"         => isset($params["NRORG"]) ? $params["NRORG"] : null,
            "CDFUNC"        => isset($params["CDFUNC"]) ? $params["CDFUNC"] : null,
            "NRVERPROPFAS"  => isset($params["NRVERPROPFAS"]) ? $params["NRVERPROPFAS"] : null,
            "NRPROPOSFAS"   => isset($params["NRPROPOSFAS"]) ? $params["NRPROPOSFAS"] : null,
            "DTSERVPREST"   => isset($params["DTSERVPREST"]) ? $params["DTSERVPREST"] : null
        );

        return $this->CdaFactory->getTipoAtividadeCda($dataParams);
    }

    public function getProspectCda()
    {
        return $this->CdaFactory->getProspectCda();
    }

    public function getInterlocutorProspectCda($params)
    {
        $dataParams = array(
            "CDPROSPECT" => isset($params["CDPROSPECT"]) ? $params["CDPROSPECT"] : null
        );
        return $this->CdaFactory->getInterlocutorProspectCda($dataParams);
    }

    public function getInterlocutorClienteCda($params)
    {
        $dataParams = array(
            "CDCLIENTE" => isset($params["CDCLIENTE"]) ? $params["CDCLIENTE"] : null,
        );
        return $this->CdaFactory->getInterlocutorClienteCda($dataParams);
    }

    public function getLicencaUsoCda($params)
    {
        $dataParams = array(
            "CDCLIENTE" => isset($params["CDCLIENTE"]) ? $params["CDCLIENTE"] : null,
        );
        return $this->CdaFactory->getLicencaUsoCda($dataParams);
    }

    public function getMotivoEvoContCda()
    {
        return $this->CdaFactory->getMotivoEvoContCda();
    }

    public function getContatosParceiroCda($params)
    {
        return $this->CdaFactory->getContatosParceiroCda(array(
            "NRPARCNEGOCIO" => isset($params["NRPARCNEGOCIO"]) ? $params["NRPARCNEGOCIO"] : null,
        ));
    }

    public function getMotivoContatoCda()
    {
        return $this->CdaFactory->getMotivoContatoCda();
    }

    public function getSistemaCda()
    {
        return $this->CdaFactory->getSistemaCda();
    }

    public function gravaContatoCDA($parametrosContato, $parametrosCda)
    {
        $nrorg = $this->Environment->getNrorg();
        $cdoperador = $this->Environment->getCdOperador();

        try {
            $contato = '';
            $novoCodigoNrseqevolcont = '';

            if (is_null($parametrosCda["NRCONTATO"]) || $parametrosCda["NRCONTATO"] == '') {
                // Gravação na tabela CONTATO
                $novoCodigo = $this->newCode->getCodeByTableName("CONTATO", $nrorg, 1, 10, true);

                $this->CdaFactory->inserirContatoCda(array(
                    "NRCONTATO"         => $novoCodigo,
                    "CDMOTICONT"        => $parametrosContato["CDMOTICONT"],
                    "CDCLIENTE"         => $parametrosCda["CDCLIENTE"],
                    "CDPROSPECT"        => $parametrosCda["CDPROSPECT"],
                    "DTHRCONTATO"       => $parametrosCda["DTSERVPREST"],
                    "DSREFECONT"        => ($parametrosCda["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE') ? "Relacionamento com o cliente" : "Visita a Prospect",
                    "IDSTATUSCONT"      => 'A',
                    "IDTPCONTATO"       => ($parametrosCda["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE') ? 'C' : 'P',
                    "CDSISTEMA"         => $parametrosContato["CDSISTEMA"],
                    "CDOPERCONT"        => $cdoperador,
                    "NRORG"             => $nrorg,
                    "NRORGINCLUSAO"     => $nrorg,
                    "NRORGULTATU"       => $nrorg,
                    "CDOPERINCLUSAO"    => $cdoperador,
                    "NRPARCNEGOCIO"     => $parametrosContato["NRPARCNEGOCIO"],
                    "NRSEQCONTATO"      => $novoCodigo,
                ));
                $contato = $novoCodigo;
            } else {
                $this->CdaFactory->atualizarContatoCda(array(
                    "NRCONTATO"         => $parametrosCda["NRCONTATO"],
                    "CDMOTICONT"        => $parametrosContato["CDMOTICONT"],
                    "CDCLIENTE"         => $parametrosCda["CDCLIENTE"],
                    "CDPROSPECT"        => $parametrosCda["CDPROSPECT"],
                    "DSREFECONT"        => ($parametrosCda["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE') ? "Relacionamento com o cliente" : "Visita a Prospect",
                    "IDTPCONTATO"       => ($parametrosCda["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE') ? 'C' : 'P',
                    "CDSISTEMA"         => $parametrosContato["CDSISTEMA"],
                    "CDOPERCONT"        => $cdoperador,
                    "CDOPERULTATU"      => $cdoperador,
                    "NRORG"             => $nrorg,
                    "NRORGULTATU"       => $nrorg,
                    "NRPARCNEGOCIO"     => $parametrosContato["NRPARCNEGOCIO"],
                    "NRSEQCONTATO"      => $parametrosCda["NRCONTATO"],
                ));
            }

            if ($parametrosCda["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE') {
                $nrrelacionaparc = $this->CdaFactory->getNrrelacionaparcCliente(array(
                    "CDCLIENTE"     => $parametrosCda["CDCLIENTE"],
                    "CDINTECLIE"    => $parametrosCda["CDINTECLIE"],
                    "NRORG"         => $nrorg
                ));
            } else {
                $nrrelacionaparc = $this->CdaFactory->getNrrelacionaparcProspect(array(
                    "CDPROSPECT"    => $parametrosCda["CDPROSPECT"],
                    "CDINTEPROS"    => $parametrosCda["CDINTEPROS"],
                    "NRORG"         => $nrorg
                ));
            }

            //Gravação na tabela EVOCONT
            if (is_null($parametrosCda["NRSEQEVOLCONT"]) || $parametrosCda["NRSEQEVOLCONT"] == '') {
                $evolcontprox = $this->CdaFactory->consultaProxSequencialEvolCont(array(
                    'NRORG'        => $nrorg,
                    'NRCONTATO'    => ($parametrosCda['NRCONTATO'] === null) ? $contato : $parametrosCda['NRCONTATO'],
                ));

                $novoCodigoNrseqevolcont = $this->newCode->getCodeByTableName("EVOLCONT", $nrorg, 1, 10, true);

                $this->CdaFactory->inserirEvoContCda(array(
                    "NRCONTATO"         => ($parametrosCda['NRCONTATO'] === null) ? $contato : $parametrosCda['NRCONTATO'],
                    "CDTIPOCONT"        => ($parametrosCda["DTHREVOLCONT"] === null) ? '60' : null, // '60' - Visita
                    "CDPROSPECT"        => $parametrosCda["CDPROSPECT"],
                    "CDINTEPROS"        => $parametrosCda["CDINTEPROS"],
                    "CDCLIENTE"         => $parametrosCda["CDCLIENTE"],
                    "CDINTECLIE"        => $parametrosCda["CDINTECLIE"],
                    "IDSENEVOCONT"      => ($parametrosCda["DTHREVOLCONT"] === null) ? 'S' : null, // 'S' - Saída
                    "IDTPEVOLCONT"      => ($parametrosCda["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE') ? 'C' : 'P',
                    "DSEVOLCONT"        => $parametrosContato["DSEVOLCONT"],
                    "IDDESCPROT"        => 'N',
                    "CDFASECONT"        => ($parametrosCda["DTHREVOLCONT"] === null) ? '036' : null, // '036' - Visita comercial
                    "IDAGEEVOCONT"      => 'N',
                    "CDOPEREVOL"        => $cdoperador,
                    "NRSEQEVOLCON"      => $evolcontprox,
                    "CDMOTIEVOCON"      => $parametrosContato["CDMOTIEVOCON"],
                    "NRORG"             => $nrorg,
                    "NRORGINCLUSAO"     => $nrorg,
                    "CDOPERINCLUSAO"    => $cdoperador,
                    "NRORGULTATU"       => $nrorg,
                    "CDOPERULTATU"      => $cdoperador,
                    "NRRELACIONAPARC"   => $nrrelacionaparc['NRRELACIONAPARC'],
                    "IDANEXOEVOLCONT"   => 'N',
                    "TXEVOLCONT"        => $parametrosContato["DSEVOLCONT"],
                    "IDATIVO"           => 'S',
                    "NRSEQEVOLCONT"     => $novoCodigoNrseqevolcont,
                    "NRSEQCONTATO"      => ($parametrosCda['NRCONTATO'] === null) ? $contato : $parametrosCda['NRCONTATO'],
                    "IDGERAFATURA"      => 'NAO',
                    "IDEVOLPUBLICO"     => 'S',
                    "IDRESPNOTA"        => 'R',
                    "IDEVOLINTERNA"     => 'S',
                    "DTHRINCLEVOCONT"   => $parametrosCda["DTSERVPREST"],
                ));
            } else {
                $this->CdaFactory->atualizarEvoLContCda(array(
                    "NRCONTATO"         => ($parametrosCda['NRCONTATO'] === null) ? $contato : $parametrosCda['NRCONTATO'],
                    "CDPROSPECT"        => $parametrosCda["CDPROSPECT"],
                    "CDINTEPROS"        => $parametrosCda["CDINTEPROS"],
                    "CDCLIENTE"         => $parametrosCda["CDCLIENTE"],
                    "CDINTECLIE"        => $parametrosCda["CDINTECLIE"],
                    "IDTPEVOLCONT"      => ($parametrosCda["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE') ? 'C' : 'P',
                    "DSEVOLCONT"        => $parametrosContato["DSEVOLCONT"],
                    "CDOPEREVOL"        => $cdoperador,
                    "CDMOTIEVOCON"      => $parametrosContato["CDMOTIEVOCON"],
                    "NRORG"             => $nrorg,
                    "NRORGULTATU"       => $nrorg,
                    "CDOPERULTATU"      => $cdoperador,
                    "NRRELACIONAPARC"   => $nrrelacionaparc['NRRELACIONAPARC'],
                    "TXEVOLCONT"        => $parametrosContato["DSEVOLCONT"],
                    "NRSEQEVOLCONT"     => $parametrosCda["NRSEQEVOLCONT"],
                    "NRSEQCONTATO"      => ($parametrosCda['NRCONTATO'] === null) ? $contato : $parametrosCda['NRCONTATO'],
                    "DTHRINCLEVOCONT"   => $parametrosCda["DTSERVPREST"],
                ));
            }

            $parametroContatoEvolcont = array(
                "NRSEQCONTATO"  => ($parametrosCda['NRCONTATO'] === null) ? $contato : $parametrosCda['NRCONTATO'],
                "NRSEQEVOLCONT" => ($novoCodigoNrseqevolcont === '') ? $parametrosCda["NRSEQEVOLCONT"] : $novoCodigoNrseqevolcont,
            );

            return $this->CdaFactory->getEvoltcontCda($parametroContatoEvolcont);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getDefaultFieldValues($idservprest)
    {
        $defaultvalues = array(
            "CDTIPOSERV"    => null,
            "CDCLIENTE"     => null,
        );

        if ($idservprest === "ATIVIDADE_INTERNA" || $idservprest === "FALTA_JUSTIFICADA" ||
            $idservprest === "FALTA_NAO_JUSTIFICADA" || $idservprest === "AFASTAMENTO_INSS" ||
            $idservprest === "FERIAS" || $idservprest === "VISITA_PROSPECT" ||  $idservprest === "RELACIONAMENTO_CLIENTE"
        ) {
            $defaultvalues["CDTIPOSERV"] = "030";
            if ($idservprest != "RELACIONAMENTO_CLIENTE") {
                $defaultvalues["CDCLIENTE"]  = "J26269316000177";
            }
        }
        return $defaultvalues;
    }

    public function formatarHorario($horario)
    {
        $horarioSemCaracteres = FindService::onlyNumber($horario);

        if (strlen($horarioSemCaracteres) == 4) {
            $hora = substr($horarioSemCaracteres, 0, 2);
            $minuto = substr($horarioSemCaracteres, 2, 2);
            return $hora . ':' . $minuto;
        }
        return null;
    }

    public function inserirAtividadeCda($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $cdoperador = $this->Environment->getCdOperador();
            $novaCda = false;
            $valoresPadrao = $this->getDefaultFieldValues($parametros["IDSERVPREST"]);

            // Criação do array da tabela SERVPREST que será usado em comum para a inserção e atualização

            $parametrosCda = array(
                "CDEMPRESA"           => $parametros["CDEMPRESA"],
                "CDFUNC"              => $parametros["CDFUNC"],
                "CDCLIENTE"           => isset($parametros["CDCLIENTE"]) ? $parametros["CDCLIENTE"] : $valoresPadrao['CDCLIENTE'],
                "DTSERVPREST"         => $parametros["DTSERVPREST"],
                "NRSERVPREST"         => $parametros["NRSERVPREST"],
                "CDTIPOSERV"          => isset($parametros["CDTIPOSERV"]) ? $parametros["CDTIPOSERV"] : $valoresPadrao['CDTIPOSERV'],
                "DSREFERENCIA"        => isset($parametros["DSREFERENCIA"]) ? $parametros["DSREFERENCIA"] : null,
                "IDSERVPREST"         => isset($parametros["IDSERVPREST"]) ? $parametros["IDSERVPREST"] : null,
                "IDSITSERVPREST"      => isset($parametros["IDSITSERVPREST"]) ? $parametros["IDSITSERVPREST"] : null,
                "CDINTECLIE"          => isset($parametros["CDINTECLIE"]) ? $parametros["CDINTECLIE"] : null,
                "DTHRENTR1TUR"        => $parametros["DTSERVPREST"],
                "DTHRSAID1TUR"        => $parametros["DTSERVPREST"],
                "DTHRENTR2TUR"        => $parametros["DTSERVPREST"],
                "DTHRSAID2TUR"        => $parametros["DTSERVPREST"],
                "NMINTCLIENTE"        => isset($parametros["NMINTCLIENTE"]) ? $parametros["NMINTCLIENTE"] : null,
                "NRPROPOSFAS"         => isset($parametros["NRPROPOSFAS"]) ? $parametros["NRPROPOSFAS"] : null,
                "NRVERPROPFAS"        => isset($parametros["NRVERPROPFAS"]) ? $parametros["NRVERPROPFAS"] : null,
                "NRSEQAGESERV"        => isset($parametros["NRSEQAGESERV"]) ? $parametros["NRSEQAGESERV"] : null,
                "NRSERVCONT"          => isset($parametros["NRSERVCONT"]) ? $parametros["NRSERVCONT"] : null,
                "CDFILIAL"            => null,
                "CDPROJETO"           => isset($parametros["CDPROJETO"]) ? ($parametros["CDPROJETO"]) : null,
                "NRORG"               => $nrorg,
                "NRORGINCLUSAO"       => $nrorg,
                "CDOPERINCLUSAO"      => $cdoperador,
                "NRORGULTATU"         => $nrorg,
                "CDOPERULTATU"        => $cdoperador,
                "CDPROSPECT"          => isset($parametros["CDPROSPECT"]) ? $parametros["CDPROSPECT"] : null,
                "NRMOTIVOEXCEFREQ"    => isset($parametros["NRMOTIVOEXCEFREQ"]) ? $parametros["NRMOTIVOEXCEFREQ"] : null,
                "QTHRTOTAL"           => isset($parametros["QTHRTOTAL"]) ? FindService::onlyNumber($parametros["QTHRTOTAL"]) : null,
                "QTHRATIVIDADEFAT"    => isset($parametros["QTHRATIVIDADEFAT"]) ? FindService::onlyNumber($parametros["QTHRATIVIDADEFAT"]) : null,
                "QTHRDESLOCAMENTOFAT" => isset($parametros["QTHRDESLOCAMENTOFAT"]) ? FindService::onlyNumber($parametros["QTHRDESLOCAMENTOFAT"]) : null,
                "DTENTRADA"           => isset($parametros['DTENTRADA']) ? $parametros["DTSERVPREST"] . ' ' . $this->formatarHorario($parametros['DTENTRADA']) : null,
                "HRAUSENCIAALMOCO"    => isset($parametros["HRAUSENCIAALMOCO"]) ? FindService::onlyNumber($parametros["HRAUSENCIAALMOCO"]) : null,
                "DTSAIDA"             => isset($parametros['DTSAIDA']) ? $parametros["DTSERVPREST"] . ' ' . $this->formatarHorario($parametros['DTSAIDA']) : null,
                "NRCONTATO"           => isset($parametros["NRSEQCONTATO"]) ? $parametros["NRSEQCONTATO"] : null,
                "DTHREVOLCONT"        => null,
                "CDINTEPROS"          => isset($parametros["CDINTEPROS"]) ? $parametros["CDINTEPROS"] : null,
                "DTHRENTR1FAT"        => $parametros["DTSERVPREST"],
                "DTHRSAID1FAT"        => $parametros["DTSERVPREST"],
                "DTHRENTR2FAT"        => $parametros["DTSERVPREST"],
                "DTHRSAID2FAT"        => $parametros["DTSERVPREST"],
                "NRLICENCAUSO"        => isset($parametros["NRLICENCAUSO"]) ? $parametros["NRLICENCAUSO"] : null,
                "NRSEQEVOLCONT"       => isset($parametros["NRSEQEVOLCONT"]) ? $parametros["NRSEQEVOLCONT"] : null,
                "DSJUSTHRNAOFATURADA" => isset($parametros["DSJUSTHRNAOFATURADA"]) ? $parametros["DSJUSTHRNAOFATURADA"] : null,
            );

            if (is_null($parametros["NRSERVPREST"]) || $parametros["NRSERVPREST"] == '' || $parametros["NRSERVPREST"] == '00000') {
                $nrservprest = $this->CdaFactory->consultaProxNrServPrest(array(
                    "NRORG"         => $nrorg,
                    "CDEMPRESA"     => $parametros["CDEMPRESA"],
                    "CDFUNC"        => $parametros["CDFUNC"],
                    "CDCLIENTE"     => isset($parametros["CDCLIENTE"]) ? $parametros["CDCLIENTE"] : $valoresPadrao['CDCLIENTE'],
                    "DTSERVPREST"   => $parametros["DTSERVPREST"]
                ));

                // Atualizando o NRSERVPREST tanto no $parametros - que são os dados puros vindo dos front, quanto no $parametrosCda que é a construção para o insert.
                $parametros['NRSERVPREST'] = $nrservprest;
                $parametrosCda['NRSERVPREST'] = $nrservprest;
                $novaCda = true;
            }

            if ($parametros["IDSERVPREST"] == 'ATIVIDADE_CLIENTE') {
                if ($novaCda) {
                    $this->CdaFactory->inserirAtividadeCda($parametrosCda);
                } else {
                    unset($parametrosCda['NRORGINCLUSAO'], $parametrosCda['CDOPERINCLUSAO']);
                    $this->CdaFactory->atualizarAtividadeCda($parametrosCda);
                }

                if (!is_null($parametros['PARTICIPANTES'])) {
                    $this->inserirOuAlterarParticipantes($parametros);
                }

                $this->inserirOuAlterarHorasNaoFaturadas($parametros);

                try {
                    $this->inserirOuAlterarDescricao($parametros, $novaCda);
                } catch (\Exception $e) {
                    throw $e;
                }
            } elseif ($parametros["IDSERVPREST"] == 'VISITA_PROSPECT' || $parametros["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE') {
                $tiposervdefault = $this->getDefaultFieldValues($parametros["IDSERVPREST"]);

                $parametrosContato = array(
                    "CDMOTICONT"    => $parametros["CDMOTICONT"],
                    "CDMOTIEVOCON"  => $parametros["CDMOTIEVOCON"],
                    "CDSISTEMA"     => $parametros["CDSISTEMA"],
                    "CDTIPOSERV"    => $tiposervdefault["CDTIPOSERV"],
                    "NRPARCNEGOCIO" => $parametros["NRPARCNEGOCIO"],
                    "DSEVOLCONT"    => $parametros["DSEVOLCONT"]
                );

                $dadosContato = $this->gravaContatoCDA($parametrosContato, $parametrosCda);

                $parametrosCda['DTHREVOLCONT'] = $dadosContato["DTHREVOLCONT"];
                $parametrosCda['NRCONTATO'] = ($parametrosCda['NRCONTATO'] === null) ? $dadosContato["NRSEQCONTATO"] : $parametrosCda['NRCONTATO'];
                $parametrosCda['NRSEQEVOLCONT'] = $dadosContato["NRSEQEVOLCONT"];

                if ($novaCda) {
                    $this->CdaFactory->inserirAtividadeCda($parametrosCda);
                } else {
                    unset($parametrosCda['NRORGINCLUSAO'], $parametrosCda['CDOPERINCLUSAO']);
                    $this->CdaFactory->atualizarAtividadeCda($parametrosCda);
                }

                if ($parametros["IDSERVPREST"] == 'VISITA_PROSPECT') {
                    $this->CdaFactory->atualizarInfoInterlocutorProspect(array(
                        "NRTEL1INPROS"    => $parametros["NRTEL1INPROS"],
                        "DSEMAILIPROS"    => $parametros["DSEMAILIPROS"],
                        "CDPROSPECT"      => $parametros["CDPROSPECT"],
                    ));
                } elseif ($parametros["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE') {
                    $this->CdaFactory->atualizarInfoInterlocutorCliente(array(
                        "NRTEL1INCLIE"    => $parametros["NRTEL1INCLIE"],
                        "DSEMAILICLIE"    => $parametros["DSEMAILICLIE"],
                        "CDINTECLIE"      => $parametros["CDINTECLIE"],
                    ));
                }
            } elseif ($parametros["IDSERVPREST"] == 'ATIVIDADE_INTERNA' || $parametros["IDSERVPREST"] == 'FERIAS' || $parametros["IDSERVPREST"] == 'FALTA_JUSTIFICADA' || $parametros["IDSERVPREST"] == 'AFASTAMENTO_INSS' || $parametros["IDSERVPREST"] == 'FALTA_NAO_JUSTIFICADA') {
                if ($novaCda) {
                    $this->CdaFactory->inserirAtividadeCda($parametrosCda);
                } else {
                    unset($parametrosCda['NRORGINCLUSAO'], $parametrosCda['CDOPERINCLUSAO']);
                    $this->CdaFactory->atualizarAtividadeCda($parametrosCda);
                }
            } elseif ($parametros['IDSERVPREST'] === 'FALTA_NAO_JUSTIFICADA') {
                if ($novaCda) {
                    $this->ProcedureService->executarFpaCdaTotaliza($nrorg, $parametrosCda["CDEMPRESA"], $parametrosCda["CDFUNC"], $parametrosCda["DTSERVPREST"]);
                }
            }

            // Busca o modelo de documento ativo
            $nrModeloDocCda = $this->CdaFactory->getModeloDocumentoCda();

            if (!is_null($nrModeloDocCda)) {
                $this->inserirOuAlterarServPrestDoc($parametros, $nrModeloDocCda);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function inserirOuAlterarServPrestDoc($parametros, $nrModeloDocCda)
    {
        try {
            $nrorg = $this->Environment->getNrorg();
            $cdoperador = $this->Environment->getCdOperador();
            $valoresPadrao = $this->getDefaultFieldValues($parametros["IDSERVPREST"]);

            $dadosDoc = array(
                "NRSERVPRESTDOC"      => null,
                "CDEMPRESA"           => $parametros["CDEMPRESA"],
                "CDFUNC"              => $parametros["CDFUNC"],
                "CDCLIENTE"           => isset($parametros["CDCLIENTE"]) ?  $parametros["CDCLIENTE"] : $valoresPadrao['CDCLIENTE'],
                "DTSERVPREST"         => $parametros["DTSERVPREST"],
                "NRSERVPREST"         => $parametros["NRSERVPREST"],
                "NRMODELODOCCDA"      => $nrModeloDocCda,
                "NRORG"               => $nrorg,
                "NRORGINCLUSAO"       => $nrorg,
                "CDOPERINCLUSAO"      => $cdoperador,
                "NRORGULTATU"         => $nrorg,
                "CDOPERULTATU"        => $cdoperador,
            );

            $inserido = $this->CdaFactory->verificaModeloDocJaInserido(array(
                "CDEMPRESA"           => $parametros["CDEMPRESA"],
                "CDFUNC"              => $parametros["CDFUNC"],
                "CDCLIENTE"           => isset($parametros["CDCLIENTE"]) ?  $parametros["CDCLIENTE"] : $valoresPadrao['CDCLIENTE'],
                "DTSERVPREST"         => $parametros["DTSERVPREST"],
                "NRSERVPREST"         => $parametros["NRSERVPREST"],
            ));


            //O valor vindo da query é um float, para isso, transformamos ele em inteiro
            if ($inserido === null) {
                $dadosDoc['NRSERVPRESTDOC'] = $this->newCode->getCodeByTableName("SERVPRESTDOC", $nrorg, 1, 10, true);
                unset($dadosDoc['NRORGULTATU'], $dadosDoc['CDOPERULTATU']);
                $this->CdaFactory->inserirServPrestDoc($dadosDoc);
            } else {
                $dadosDoc['NRSERVPRESTDOC'] = $inserido['NRSERVPRESTDOC'];
                unset($dadosDoc['NRORGINCLUSAO'], $dadosDoc['CDOPERINCLUSAO']);
                $this->CdaFactory->alterarServPrestDoc($dadosDoc);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function inserirOuAlterarParticipantes($parametros)
    {
        try {
            $nrorg = $this->Environment->getNrorg();
            $cdoperador = $this->Environment->getCdOperador();
            $valoresPadrao = $this->getDefaultFieldValues($parametros["IDSERVPREST"]);

            foreach ($parametros['PARTICIPANTES'] as $participante) {
                $dadosParticipante = array(
                    "NRSERVPRESTPARTICIP" => $participante["NRSERVPRESTPARTICIP"],
                    "DSNOMEPARTICIP"      => $participante["DSNOMEPARTICIP"],
                    "DSSETORPARTICIP"     => isset($participante["DSSETORPARTICIP"]) ? $participante["DSSETORPARTICIP"] : null,
                    "CDEMPRESA"           => $parametros["CDEMPRESA"],
                    "CDFUNC"              => $parametros["CDFUNC"],
                    "CDCLIENTE"           => isset($parametros["CDCLIENTE"]) ?  $parametros["CDCLIENTE"] : $valoresPadrao['CDCLIENTE'],
                    "DTSERVPREST"         => $parametros["DTSERVPREST"],
                    "NRSERVPREST"         => $parametros["NRSERVPREST"],
                    "NRORG"               => $nrorg,
                    "NRORGINCLUSAO"       => $nrorg,
                    "CDOPERINCLUSAO"      => $cdoperador,
                    "NRORGULTATU"         => $nrorg,
                    "CDOPERULTATU"        => $cdoperador,
                );

                // Verifica se o participante já foi inserido
                $inserido = $this->CdaFactory->verificarParticipanteJaInserido(array(
                    "NRSERVPRESTPARTICIP" => $participante["NRSERVPRESTPARTICIP"],
                ));

                if (intval($inserido) === 0) {
                    $dadosParticipante['NRSERVPRESTPARTICIP'] = $this->newCode->getCodeByTableName("SERVPRESTPARTICIP", $nrorg, 1, 10, true);
                    unset($dadosParticipante['NRORGULTATU'], $dadosParticipante['CDOPERULTATU']);
                    $this->CdaFactory->inserirParticipantesCda($dadosParticipante);
                } else {
                    unset($dadosParticipante['NRORGINCLUSAO'], $dadosParticipante['CDOPERINCLUSAO']);
                    $this->CdaFactory->atualizarParticipantes($dadosParticipante);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function inserirOuAlterarDescricao($parametros, $novaCda)
    {
        try {
            $nrorg = $this->Environment->getNrorg();
            $cdoperador = $this->Environment->getCdOperador();

            $paramsDesc = array(
                "DSSERVPREST"         => $parametros["DESCSERVPRESS"],
                "CDEMPRESA"           => $parametros["CDEMPRESA"],
                "CDFUNC"              => $parametros["CDFUNC"],
                "CDCLIENTE"           => $parametros["CDCLIENTE"],
                "DTSERVPREST"         => $parametros["DTSERVPREST"],
                "NRSERVPREST"         => $parametros["NRSERVPREST"],
                "NRORG"               => $nrorg,
                "NRORGINCLUSAO"       => $nrorg,
                "CDOPERINCLUSAO"      => $cdoperador,
                "NRORGULTATU"         => $nrorg,
                "CDOPERULTATU"        => $cdoperador,
            );

            if ($novaCda) {
                unset($paramsDesc['NRORGULTATU'], $paramsDesc['CDOPERULTATU']);
                $this->CdaFactory->inserirDescricaoCDA($paramsDesc);
            } else {
                unset($paramsDesc['CDOPERINCLUSAO'], $paramsDesc['CDOPERINCLUSAO']);
                $this->CdaFactory->atualizarDescricaoCDA($paramsDesc);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getDescricaoCda($parametros)
    {
        try {
            $nrorg = $this->Environment->getNrorg();

            $data = $this->CdaFactory->getDescricaoCda(array(
                "CDEMPRESA"           => $parametros["CDEMPRESA"],
                "CDFUNC"              => $parametros["CDFUNC"],
                "CDCLIENTE"           => $parametros["CDCLIENTE"],
                "DTSERVPREST"         => $parametros["DTSERVPREST"],
                "NRSERVPREST"         => $parametros["NRSERVPREST"],
                "NRORG"               => $nrorg,
            ));

            foreach ($data as &$row) {
                $row['DSSERVPREST'] = mb_check_encoding($row['DSSERVPREST'], 'UTF-8') ? $row['DSSERVPREST'] :  utf8_encode($row['DSSERVPREST']);
            }
            return $data;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getParticipantesCda($parametros)
    {
        return $this->CdaFactory->getParticipantesCda(array(
            "CDEMPRESA"           => $parametros["CDEMPRESA"],
            "CDFUNC"              => $parametros["CDFUNC"],
            "CDCLIENTE"           => $parametros["CDCLIENTE"],
            "DTSERVPREST"         => $parametros["DTSERVPREST"],
            "NRSERVPREST"         => $parametros["NRSERVPREST"],
        ));
    }

    public function deleteParticipantes($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $this->CdaFactory->deleteParticipantes(FindService::formatarArray2($parametros['NRSERVPRESTPARTICIP']));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function inserirOuAlterarHorasNaoFaturadas($parametros)
    {
        try {
            $nrorg = $this->Environment->getNrorg();
            $cdoperador = $this->Environment->getCdOperador();
            $valoresPadrao = $this->getDefaultFieldValues($parametros["IDSERVPREST"]);

            foreach ($parametros['HORASNFATURADAS'] as $horaNFatu) {
                $hora_sem_dois_pontos = str_replace(":", "", $horaNFatu["QTHORASNAOFATU"]);
                $hora_formatada = str_pad($hora_sem_dois_pontos, 4, '0', STR_PAD_LEFT);

                $dados = array(
                    "NRSERVPRESTJUSTHRNFAT"     => $horaNFatu['NRSERVPRESTJUSTHRNFAT'],
                    "CDEMPRESA"                 => $parametros["CDEMPRESA"],
                    "CDFUNC"                    => $parametros["CDFUNC"],
                    "CDCLIENTE"                 => isset($parametros["CDCLIENTE"]) ?  $parametros["CDCLIENTE"] : $valoresPadrao['CDCLIENTE'],
                    "DTSERVPREST"               => $parametros["DTSERVPREST"],
                    "NRSERVPREST"               => $parametros["NRSERVPREST"],
                    "QTHORASNAOFATU"            => $hora_formatada,
                    "NRJUSTHORASNAOFATUCDA"     => $horaNFatu["NRJUSTHORASNAOFATUCDA"],
                    "NRORG"                     => $nrorg,
                    "CDOPERINCLUSAO"            => $cdoperador,
                    "NRORGINCLUSAO"             => $nrorg,
                    "NRORGULTATU"               => $nrorg,
                    "CDOPERULTATU"              => $cdoperador,
                );

                if ($horaNFatu['NRSERVPRESTJUSTHRNFAT'] === 0) {
                    // $nr = $this->CdaFactory->getProxNrHorasNaoFaturadas();
                    $dados['NRSERVPRESTJUSTHRNFAT'] = $this->newCode->getCodeByTableName("SERVPRESTJUSTHRNFAT", $nrorg, 1, 10, true);
                    unset($dados['NRORGULTATU'], $dados['CDOPERULTATU']);
                    $this->CdaFactory->inserirHorasNaoFaturadas($dados);
                } else {
                    unset($dados['NRORGINCLUSAO'], $dados['CDOPERINCLUSAO']);
                    $this->CdaFactory->atualizarHorasNaoFaturadas($dados);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function gerarDocumento($parametros)
    {
        // parametros = dados da cda e idoma do documento
        $dadosDocumento = $this->CdaFactory->buscarDocumento(array(
            "CDEMPRESA"           => $parametros["CDEMPRESA"],
            "CDFUNC"              => $parametros["CDFUNC"],
            "CDCLIENTE"           => $parametros["CDCLIENTE"],
            "DTSERVPREST"         => $parametros["DTSERVPREST"],
            "NRSERVPREST"         => $parametros["NRSERVPREST"],
            "CDIDIOM"             => $parametros["CDIDIOM"],
        ));
    }

    public function getHorasNaoFaturadas($parametros)
    {
        return $this->CdaFactory->getHorasNaoFaturadas(array(
            "CDEMPRESA"           => $parametros["CDEMPRESA"],
            "CDFUNC"              => $parametros["CDFUNC"],
            "CDCLIENTE"           => $parametros["CDCLIENTE"],
            "DTSERVPREST"         => $parametros["DTSERVPREST"],
            "NRSERVPREST"         => $parametros["NRSERVPREST"],
        ));
    }

    public function existenciaCda($parametros)
    {
        return $this->CdaFactory->existenciaCda(array(
            "DTSERVPREST"         => $parametros["DTSERVPREST"],
        ));
    }

    public function confirmaOuCorrigeCda($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrorg();
            $cdoperador = $this->Environment->getCdOperador();

            $this->CdaFactory->confirmaOuCorrigeCda(array(
                "CDEMPRESA"                 => $parametros["CDEMPRESA"],
                "CDFUNC"                    => $parametros["CDFUNC"],
                "CDCLIENTE"                 => $parametros["CDCLIENTE"],
                "DTSERVPREST"               => $parametros["DTSERVPREST"],
                "NRSERVPREST"               => $parametros["NRSERVPREST"],
                "IDSITSERVPREST"            => ($parametros['ACAO'] === 'confirmar') ?  'C' : 'P',
                "NRORG"                     => $nrorg,
                "NRORGULTATU"               => $nrorg,
                "CDOPERULTATU"              => $cdoperador,
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function deletarCda($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrorg();
            foreach ($parametros as $value) {
                $dados = array(
                    "CDEMPRESA"                 => $value["CDEMPRESA"],
                    "CDFUNC"                    => $value["CDFUNC"],
                    "CDCLIENTE"                 => $value["CDCLIENTE"],
                    "DTSERVPREST"               => $value["DTSERVPREST"],
                    "NRSERVPREST"               => $value["NRSERVPREST"],
                );

                $this->CdaFactory->deleteBatidaPonto($dados);
                $this->CdaFactory->deleteDescServPrest($dados);
                $this->CdaFactory->deleteServPrestDoc($dados);
                
                if ($value['IDSERVPREST'] === 'ATIVIDADE_CLIENTE') {
                    $this->CdaFactory->deleteParticipantesCda($dados);
                    $this->CdaFactory->deleteHorasNaoFaturadas($dados);

                } elseif ($value['IDSERVPREST'] === 'VISITA_PROSPECT' || $value['IDSERVPREST'] === 'RELACIONAMENTO_CLIENTE') {
                    $this->CdaFactory->deleteEvolCont(array(
                        "NRSEQEVOLCONT" => $value['NRSEQEVOLCONT'],
                        "NRCONTATO"     => $value['NRCONTATO'],
                        "DTHREVOLCONT"  => $value['DTHREVOLCONT'],
                    ));
                } elseif ($value['IDSERVPREST'] === 'FALTA_NAO_JUSTIFICADA') {
                    $this->ProcedureService->executarFpaCdaTotaliza($nrorg, $value["CDEMPRESA"], $value["CDFUNC"], $value["DTSERVPREST"]);
                }

                $this->CdaFactory->deleteServPrest($dados);
            }
            
            $this->connection->commit();
        } catch (\Exception $e) {
            throw $e;
        }
    }
}
